// Bankrun based on Gertler and  Kiyotaki  2015 with housing and borrowers
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 var 
// Lagrangians,*******1-3
lambda_P,lambda_I,lambda_m,

//consumption 4-8
C_P,C_I,C_B,C,CT,

//housing 9-11
h_P,h_I,H,

//Deposit & mortgage  12-13
D,M

//capital  14-16
K_h,K_b,K,

//wage  17-20
w_P,w_I,l_P,l_I,

//interests & price  21-24
r,r_b,p_h,p_k, 

//BANK
  // networth 25-29
N_l, N, N_1_N, A, %, N_n
  //multiplier 30-36
phi_d,phi_k,phi,s,v,Omega, q_k,

//capital producer 37
i_k,
//cost function 38-39
f_k,f_k_1,
//shock 40-41
xi,z,

// GDP 42
Y,
//adjustment cost 43-48
AC_P,AC_I,AC_B,ac_dh,ac_kh,ac_mh, %

//results49-58
data_Y,data_CT ,data_i_k ,data_A ,data_M ,data_D ,data_p_h ,data_K,spread_rb_r,data_phi;

%********************************
 varexo epsilon_xi;
%********************************

  parameters
  //beta
beta_P,beta_I,beta_B,
 // utility coeficient
varphi_c,varphi_h,varphi_l,
//depreciation
delta_k,
//probability of life
sigma,
//downpayment
theta_m,
//coeficient of default
theta_b,
//donation
varrho,
//cost function
alpha_k, N_n,
//shock
rho_z, xi_bar, 
//adjustment cost
psi_dh,psi_db,psi_mh,psi_mb,psi_kb,psi_kh,psi_k;

%********************************
// define the parameters
  //beta
beta_P = 0.961538462 ;
beta_I = 0.95446361936 ;
beta_B = 0.9606147945 ;

 // utility coeficient
varphi_c = 0.5 ;
varphi_h = 0.337278099 ; 
varphi_l = 1.5754225436 ;

//depreciation
delta_k = 22/600 ;
//probability of life
sigma = 0.877 ; %0.877
//downpayment
theta_m = 0.6976744; % M=1.56
//coeficient of default
theta_b = 0.3401746539; 
//donation
varrho = 1.6992e-04 ; %1.6992e-04
//cost function 
alpha_k = 0.00516956 ;
//shock
rho_z = 0.90 ; 
xi_bar = 0.086666666 ;
//adjustment cost
psi_dh = 0;
psi_db = 0;
psi_mh = 0;
psi_mb = 0;
psi_kb = 0;
psi_kh = 0;
psi_k  = 1.73;
//%%%%%%%% steady state
N_n = 0.001;
%********************************

model;

// Model local variables:every time this variable appears in the model, Dynare will
//substitute it by the expression assigned to the variable used outside.
#  D_bar=steady_state(D);
#  K_bar=steady_state(K);
#  M_bar=steady_state(M);
#  K_h_bar=steady_state(K_h);
#  K_b_bar=steady_state(K_b);
#  h_P_bar=steady_state(h_P);
#  h_I_bar=steady_state(h_I);
#  i_k_bar=steady_state(i_k);
#  p_h_bar=steady_state(p_h);
#  Y_bar=steady_state(Y);
#  C_bar=steady_state(C);
#  CT_bar=steady_state(CT);
#  A_bar=steady_state(A);
#  phi_bar=steady_state(phi);
%#  xi_bar=steady_state(xi);
%********************************

// Lagrangian 1-2
lambda_P = (1- varphi_c)/(C_P - varphi_c * C_P(-1)); //1 lambda_P
lambda_I = (1- varphi_c)/(C_I - varphi_c * C_I(-1)); //2 lambda_I

//Houshold budget constraints
  //lender budget 3
%C_P + D + p_h * (h_P - h_P(-1)) + p_k * (K_h-(1-delta_k)* K_h(-1)) + f_k + (1-sigma) * N_n = (1+xi) * w_P * l_P + (1+r(-1)) *D(-1) + xi * K_h(-1) - AC_P ; //3.C_P
 C_P + D + p_h * (h_P - h_P(-1)) + p_k * (K_h-(1-delta_k)* K_h(-1)) + f_k + (1-sigma) * N_n = (1+xi) * w_P * l_P + (1+r)     *D(-1) + xi * K_h(-1) - AC_P ; //3.C_P

  //borrower budget 4
%C_I + p_h * (h_I - h_I(-1)) + (1+r_b(-1))*M(-1)= (1+xi) * w_I* l_I + M - AC_I; //4 C_I
 C_I + p_h * (h_I - h_I(-1)) + (1+r_b)    *M(-1)= (1+xi) * w_I* l_I + M - AC_I; //4 C_I

//FOCs
  // Lender respect to h^p 5
p_h =  varphi_h / (lambda_P * h_P)+ beta_P *  (lambda_P(1)/ lambda_P) * p_h(1);//5 p_h

  // Lender respect to D 6
%1 + ac_dh = beta_P *  (lambda_P(1)/ lambda_P) * (1+r); //6 r
1 + ac_dh = beta_P *  (lambda_P(1)/ lambda_P) * (1+r(1)); //6 r

  // Lender respect to K^h 7
p_k + f_k_1 + ac_kh = beta_P * (lambda_P(1)/ lambda_P) * (p_k(1)* (1- delta_k)+ xi(1)); //7 K_h

  // Lender respect to l^P 8
 varphi_l/(1-l_P) = lambda_P * (1+xi) * w_P; //8 w_P

  // borrower respect to h^I 9
(1-  (lambda_m / lambda_I) * theta_m ) * p_h=  varphi_h/ (lambda_I * h_I) + beta_I * ( lambda_I(1)/ lambda_I) * p_h(1); //9 h_I

  // borrower respect to M 10
%1- (lambda_m/lambda_I) +ac_mh = beta_I *( lambda_I(1)/ lambda_I) * (1+r_b); //10  lambda_m
1- (lambda_m/lambda_I) +ac_mh = beta_I *( lambda_I(1)/ lambda_I) * (1+r_b(1)); //10  lambda_m

  // borrower respect to l^I 11
varphi_l/ (1-l_I)= lambda_I * (1+xi) * w_I; //11 w_I
 
// Aggregate banking 
   //Arbitrage 12-13
%xi + (1- delta_k)* p_k = (1+r_b(-1))* p_k(-1); //12 r_b
 xi + (1- delta_k)* p_k = (1+r_b)* p_k(-1); //12 r_b
 
A = p_k * K_b + M ;  //13 K_b
   // living networth 14
%N_l = (1+r_b(-1)) * A(-1) - (1+r(-1)) * D(-1); //14
 N_l = (1+r_b)     * A(-1) - (1+r)     * D(-1); //14

   // new networth 15
%N_n =  varrho * p_k * K  ; //15
   // Total networth 16
N = sigma * N_l + (1- sigma) * N_n; //16

   // Bnak consumption  17
C_B = (1- sigma) * N_l; //17
   // budget constraint 18
C_B + A + AC_B = D + N_l + (1- sigma) * N_n ; //18 A

   // phi 19-20
phi = A/N ; //19  phi
phi_d = D/N ;//20 D
phi_k = p_k * K_b /N; //21 phi_k
   //  22-23
%q_k =  (xi + (1- delta_k)* p_k - (1+r_b(-1))* p_k (-1))/ p_k(-1); //22
 q_k =  (xi + (1- delta_k)* p_k - (1+r_b)    * p_k (-1))/ p_k(-1); //22

s =  r_b - r ; //23
   // Omega 24
Omega = beta_B * ((1- sigma)+ sigma * v); //24
   // internal leverage ratio 25-26
%N_1_N = (q_k(1) * phi_k + s * phi_d + (1+r_b)); //25 %
 N_1_N = (q_k(1) * phi_k + s(1) * phi_d + (1+r_b(1))); //25 %
v = Omega(1) * N_1_N ; //26 v

  // tobins Q 27
v = theta_b * ( phi_d + 1) ; //27 phi_d


//Market clearing
H = h_P+h_I; //28 h_p
H = 1;  //29
K = K_b + K_h; //30
Y = xi * K(-1) + (1+xi) * (w_P * l_P + w_I * l_I);//31 Y 
Y = f_k + CT + i_k;//32 l_I
C = C_P + C_I ;//33
CT = C + C_B; //34

// colateral constraint 35
M = theta_m * p_h * h_I ; //35



// cost function 36-37
f_k= alpha_k/ 2 * (K_h)^2; //36
f_k_1= alpha_k * K_h;// 37

// capital producer 38-39
(1- psi_k * (i_k/i_k(-1) - 1)^2 /2) * i_k = K - (1-delta_k) * K(-1); //38 i_k
p_k-psi_k*p_k*(i_k/i_k(-1)-1)*(i_k/i_k(-1))-psi_k*p_k*(i_k/i_k(-1)-1)^2/2+beta_P*((lambda_P(1)/lambda_P)*psi_k*p_k(1)*(i_k(1)/i_k-1)*(i_k(1)/i_k)^2) = 1;//39



//shock 40-41
z = rho_z * z(-1) - epsilon_xi ;    //40                       %Productivity shock  
xi= xi_bar * exp(z); //41
l_P = 0.30 ; //42



// Adjustment cost 43-48
AC_P=(psi_dh/2)* (D-D(-1))^2/D_bar + (psi_kh/2)* (K_h-K_h(-1))^2/K_h_bar;//43
AC_I=(psi_mh/2)* (M-M(-1))^2/M_bar ;//44
AC_B=(psi_db/2)* (D-D(-1))^2/D_bar + (psi_kb/2)* (K_b-K_b(-1))^2/K_b_bar +(psi_mb/2)* (M-M(-1))^2/M_bar;//45
ac_dh= (psi_dh)* (D-D(-1))/D_bar ;//46
ac_kh= (psi_kh)* (K_h-K_h(-1))/K_h_bar;//47
ac_mh= (psi_mh)* (M-M(-1))/M_bar ;//48


%/*
//Results 49-28
data_Y = Y/Y_bar - 1; //49
data_CT = CT/CT_bar - 1 ; //50
data_i_k = i_k/i_k_bar - 1 ; //51
data_A = A/A_bar - 1 ; //52
data_M = M/M_bar - 1  ; //53
data_p_h = p_h/p_h_bar - 1 ; //54
spread_rb_r = r_b - r; //55
data_D = D/D_bar - 1 ; //56
data_K = K/K_bar - 1 ; //57
data_phi = phi/phi_bar - 1 ; //58
%*/
end;
%********************************

load just_for_check_0_BR.mat;
initval;
lambda_P = lambda_P_0; lambda_I= lambda_I_0;
C_P=C_P_0; C_I=C_I_0 ;C_B=C_B_0 ;C=C_0 ; CT= CT_0;
h_P=h_P_0 ;h_I=h_I_0 ;H=H_0;
l_P=l_P_0; l_I=l_I_0;  D=D_0; M=M_0; K_h=K_h_0;K_b=K_b_0; K=K_0;
w_P=w_P_0; w_I=w_I_0; r=r_0; r_b=r_b_0; p_h=p_h_0; p_k=p_k_0; 
N_l=N_l_0;  N=N_0; phi_d=phi_d_0; phi=phi_0;  phi_k=phi_k_0;  s=s_0; v=v_0; Omega=Omega_0; %q_k=q_k_0; %N_n=N_n_0;
i_k=i_k_0; f_k=f_k_0 ; xi=xi_0; A=A_0;  Y=Y_0; z=0; 
AC_P=0; AC_I=0; AC_B=0; ac_dh=0; ac_kh=0; ac_mh=0;

epsilon_xi = 0;

end;
%********************************

steady;

check;

shocks;
var epsilon_xi ; stderr 0.001;

end;

// order 1=  At first order they are deviations from the steady state.
// with relative_irf= relative_irf only normalizes the variables by the shock size so that the shock sizes are comparable. It does not rescale the other variables by their steady state values to make them percentage deviations!
// order 2 et + = % deviation 
stoch_simul(periods=2100,irf = 100,order =1);









